from time import sleep
from iot_gh.IoTGreenhouse import IoTGreenhouse
from iot_gh.IoTGreenhouseService import IoTGreenhouseService


ghs = None      #iot_greenhouse_service
gh = None       #iot_greenhouse data object

def lamp_test():
    global ghs 
    print("Testing lamps...")
    print("Note: jumper must be positioned on J1 for Red LED to light.")
    print("Light red")
    ghs.lamps.red.on() 
    sleep(2)
    ghs.lamps.red.off() 
    sleep(1)
    print("Light white")
    ghs.lamps.white.on() 
    sleep(2)
    ghs.lamps.white.off() 
    sleep(1)
    print("Light green")
    ghs.lamps.dual.on_green() 
    sleep(2)
    ghs.lamps.dual.off() 
    sleep(1)
    print("Light yellow")
    ghs.lamps.dual.on_yellow() 
    sleep(2)
    ghs.lamps.dual.off() 
    sleep(1)
    print("Lamp test done.")
    print()

def switch_test():
    global ghs 

    print("Testing switches. PB activates Red LED, Toggle activates White LED.")
    print("Switch both to on to end.")
    while(ghs.switches.push_button.get_state() == ghs.switches.SWITCH_OFF or 
          ghs.switches.toggle.get_state() == ghs.switches.SWITCH_OFF):
        
        if ghs.switches.push_button.get_state() == ghs.switches.SWITCH_ON:
            ghs.lamps.red.on()
        else:
            ghs.lamps.red.off() 
        if ghs.switches.toggle.get_state() == ghs.switches.SWITCH_ON:
            ghs.lamps.white.on()
        else:
            ghs.lamps.white.off()
    
    ghs.lamps.red.off()
    ghs.lamps.white.off()        
    print("Switch test done.")
    print()

def fan_test():
    global ghs
    
    print("Testing fan.")
    
    while(ghs.switches.toggle.get_state() == ghs.switches.SWITCH_ON):
        print("Toggle must be off for test to start.")
        sleep(1)
    print("Press push botton to activitate fan. Turn toggle on to end.")
    while(ghs.switches.toggle.get_state() == ghs.switches.SWITCH_OFF):
        if ghs.switches.push_button.get_state() == ghs.switches.SWITCH_ON:
            ghs.fan.on()
        else:
            ghs.fan.off()
    ghs.fan.off()        
    print("Fan test done.")
    print()

def servo_test():
    global ghs 

    print("Testing servo.")
    print("Servo position set in ~/iot_gh/iot_gh_system.conf.")
    while(ghs.switches.toggle.get_state() == ghs.switches.SWITCH_ON):
        print("Toggle must be off for test to start.")
        sleep(1)
    print("Servo moves between min and max Turn toggle on to end.")
    while(ghs.switches.toggle.get_state() == ghs.switches.SWITCH_OFF):
        pos = 0
        while pos > -1:
            ghs.servo.move(pos)
            pos -= .1
            sleep(.2)
        pos = -1
        while pos < 1:
            ghs.servo.move(pos)
            pos += .1
            sleep(.2)
        pos = 1
    print("Servo test done.")
    print()

def temp_sensor_test():
    global ghs 
    print("Testing temp sensor.")
    while(ghs.switches.toggle.get_state() == ghs.switches.SWITCH_ON):
        print("Toggle must be off for test to start.")
        sleep(1)
    print("Turn toggle on to end.")
    while(ghs.switches.toggle.get_state() == ghs.switches.SWITCH_OFF):
            print("i temp C: %d" % ghs.temperature.get_inside_temp_C())
            print("i temp F: %d" % ghs.temperature.get_inside_temp_F())
            print("o temp C: %d" % ghs.temperature.get_outside_temp_C())
            print("o temp F: %d" % ghs.temperature.get_outside_temp_F())
            print()
            sleep(1)
    print("Temp sensor test done.")
    print()    

def analog_test():

    print("Testing analog inputs")
    while(ghs.switches.toggle.get_state() == ghs.switches.SWITCH_ON):
        pass #wait for toggle off

    while(ghs.switches.toggle.get_state() == ghs.switches.SWITCH_OFF):
 
        print("Pot value: %i" % ghs.analog.pot.get_value())
        print("Light value:" + str(ghs.analog.light.get_value()))
        print("Aux value:" + str(ghs.analog.aux.get_value()))
        print()
        sleep(1)

def test_all():
    global ghs
    
    print("*** GH testing ***")   
    ghs = IoTGreenhouseService()
    iot_greenhouse = ghs.make_greenhouse("test")
    
    ghs.buzzer.beeps(delay=.2, beep_count = 2)
    lamp_test()
    ghs.buzzer.beeps(beep_count = 1)
    switch_test()
    ghs.buzzer.beeps(beep_count = 1)
    fan_test()
    ghs.buzzer.beeps(beep_count = 1)
    servo_test()
    ghs.buzzer.beeps(beep_count = 1)
    temp_sensor_test()
    ghs.buzzer.beeps(beep_count = 1)
    analog_test()
   
    print("GH test completed.")   

if __name__ == "__main__":
    test_all()






import pigpio

class GHLamps(object):
    """This class GHLamps provides a services to control the IoT Greenhouse LED lamps.
    """
    red = None
    white = None
    dual = None

    def __init__(self, pi, red_led_gpio, white_led_gpio, dual_led_gpio1, dual_led_gpio2):
        """Constructor for class GHLamps
        
        :param pi: reference to pigpio service
        :param red_led_gpio: GPIO port for red LED. Use GHgpio definitions.
        :param white_led_gpio: GPIO port for white LED. Use GHgpio definitions.
        :param dual_led_gpio1: GPIO port for dual LED pin 1. Use GHgpio definitions.
        :param dual_led_gpio2: GPIO port for dual LED pin 2. Use GHgpio definitions.
        :returns: GHLamps object
        """
        self.red = GHLamp(pi, red_led_gpio)
        self.white =  GHLamp(pi, white_led_gpio)
        self.dual = GHLampDual(pi, dual_led_gpio1, dual_led_gpio2)


class GHLamp(object):
    """This class GHLamp provides a services to control individual IoT Greenhouse LED lamp.
    """   
    LAMP_ON = False #active log
    LAMP_OFF = True

    _pi = None
    _gpio = 0

    def __init__(self, pi, gpio):
        """Constructor for class GHLamp
        
        :param pi: reference to pigpio service
        :param gpio: GPIO port for LED. Use GHgpio definitions.
        :returns: GHLamp object
        """
        self._pi = pi
        self._gpio = gpio
        self._pi.set_mode(self._gpio, pigpio.OUTPUT) 
        self._pi.write(self._gpio, self.LAMP_OFF)

    def on(self):
        self._pi.write(self._gpio, self.LAMP_ON)
    
    def off(self):
        self._pi.write(self._gpio, self.LAMP_OFF)

    def get_state(self):
        return self._pi.read(self._gpio)
    
    def is_on(self):
        return self.get_state() == self.LAMP_ON

    def is_off(self):
        return self.get_state() == self.LAMP_OFF
    
    def get_status(self):
        if self.is_on():
            return "ON"
        else:
            return "OFF"
            

class GHLampDual(object):
    """This class GHLamp provides a services to control dual color IoT Greenhouse LED lamp.
    """   
    LAMP_OFF = 0
    LAMP_ON_YELLOW = 1
    LAMP_ON_GREEN = 2

    _pi = None
    _shared_gpio1 = 0
    _gpio2 = 0
    _state = LAMP_OFF

    def __init__(self, pi, shared_gpio1, gpio2):
        """Constructor for class GHLampDual
        
        :param pi: reference to pigpio service
        :param shared_gpio1: GPIO port 1 for Dual LED. Currently shared with RED LED.
        :param gpio2: GPIO port 2 for Dual LED. 
        :returns: GHLampDual object
        """
        self._pi = pi
        self._shared_gpio1 = shared_gpio1
        self._gpio2 = gpio2
        self._pi.set_mode(self._shared_gpio1, pigpio.OUTPUT)   
        self._pi.set_mode(self._gpio2, pigpio.INPUT) 
        self._pi.write(self._shared_gpio1, True)
        self._state = self.LAMP_OFF

    def on_green(self):
        self._pi.set_mode(self._gpio2, pigpio.OUTPUT)
        self._pi.write(self._shared_gpio1, False)
        self._pi.write(self._gpio2, True)
        self._state = self.LAMP_ON_GREEN

    def on_yellow(self):
        self._pi.set_mode(self._gpio2, pigpio.OUTPUT)
        self._pi.write(self._shared_gpio1, True)
        self._pi.write(self._gpio2, False)
        self._state = self.LAMP_ON_YELLOW

    def off(self):
        self._pi.set_mode(self._gpio2, pigpio.INPUT)
        self._state = self.LAMP_OFF

    def get_state(self):
        return self._state

    def is_green(self):
        return self.get_state() == self.LAMP_ON_GREEN

    def is_yellow(self):
        return self.get_state() == self.LAMP_ON_YELLOW

    def is_off(self):
        return self.get_state() == self.LAMP_OFF

    def get_status(self):
        if self.is_yellow():
            return "GREEN"
        if self.is_green():
            return "YELLOW"
        else:
            return "OFF"


from iot_gh.IoTGreenhouse import IoTGreenhouse
from iot_gh.IoTGreenhouseService import IoTGreenhouseService


ghs = None      #iot_greenhouse_service
gh = None       #iot_greenhouse data object

def test_post():
    global ghs
    
    print("*** GH testing ***")   
    ghs = IoTGreenhouseService()
    gh = ghs.make_greenhouse("test")
    gh.message = "test post"
    ghs.web_service.post_greenhouse(gh)
    print("status code = %i" % gh.post_data.statuscode)
    print("GH post test completed.")   

if __name__ == "__main__":
    test_post()






from time import time
import requests
import json
import ast
 
class GHPostData():
    """Data object for post response
    """
    status_code = 0
    last_update = None
    exception = None

class GHWebService(object):
    """ Web connector for Flow service. Post data using json package.
    """
    url = None

    def __init__(self, url):
        self.url = url
        
    def post_greenhouse(self, greenhouse):
        #throttle post to every 30 sec
        if greenhouse.post_data == None or time() > greenhouse.post_data.last_update + 30:
            p_data = self._post(greenhouse)
            greenhouse.post_data = p_data

    #def _make_payload(self, greenhouse):
        
        #payload = json.dumps(greenhouse.__dict__)
        #payload = {
        #    "house_ID": greenhouse.settings.house_ID,
        #    "row_id": greenhouse.settings.row_id,
        #    "group_ID": greenhouse.settings.group_ID,
        #    "version":  greenhouse.settings.version,
        #    "name": greenhouse.name,
        #    "led_red_state": greenhouse.led_red_state,
        #    "led_white_state": greenhouse.led_white_state,
        #    "led_dual_state":  greenhouse.led_dual_state,
        #    "switch_pb_state":  greenhouse.switch_pb_state,
        #    "switch_toggle_state": greenhouse.switch_toggle_state,
        #    "fan_state": greenhouse.fan_state,
        #    "servo_position":  greenhouse.servo_position,
        #    "buzzer_state": greenhouse.buzzer_state,
        #    "ain_pot_position": greenhouse.ain_pot_position,
        #    "ain_light_raw":  greenhouse.ain_light_raw,
        #    "ain_aux_raw": greenhouse.ain_aux_raw,

        #    "temp_inside_C":  greenhouse.temp_inside_C,
        #    "temp_inside_F": greenhouse.temp_inside_F,
        #    "temp_outside_C":  greenhouse.temp_outside_C,
        #    "temp_outside_F": greenhouse.temp_outside_F,
        #    #"temp_humidity": greenhouse.temp_humidity,
        #    #"temp_reading_is_valid":  greenhouse.temp_reading_is_valid,
        #    #"temp_last_valid_reading":  greenhouse.temp_last_valid_reading,
            
        #    "valid_state": greenhouse.valid_state,
        #    "last_update": greenhouse.last_update
        #    }

        #return payload
            
    def _post(self, gh):
        '''HTTP request post of json dump

        json package formated for Flow service
        '''
        post_data = GHPostData()
        try:
            headers = {'Content-type': 'application/json', 'Accept': 'text/plain'}
            payload = json.dumps(gh.__dict__)
            json_payload = ast.literal_eval(payload)
            r = requests.post(url, json=json_payload , headers=headers)
            post_data.status_code = r.status_code
            post_data.last_update = time()
            
        except Exception as ex:
            print("Error: Unable to post data to service. Check for valid Wi-Fi connection. %s" % ex.args[0])
            post_data.exception = ex 
        
        finally:
            return post_data


def test_post():
    from iot_gh.IoTGreenhouseService import IoTGreenhouseService

    ghs = IoTGreenhouseService()
    gh = ghs.make_greenhouse("test")
    ws = GHWebService(ghs._url)
    ws.post_greenhouse(gh)
    print(gh.post_data.status_code)
    print(gh.post_data.last_update)
    print(gh.post_data.exception)
    
if __name__ == "__main__":
    test_post()


class IoTGreenhouse(object):
    """IoT Greenhouse data object"""
    #house IDs
    name = None
    house_id = None
    group_id = None
    house_number = 0
    #house state
    valid_state = False
    last_update = None
    led_red_state = None
    led_white_state = None
    led_dual_state = None
    switch_pb_state = None
    switch_toggle_state = None
    fan_state = None
    servo_position = None
    buzzer_state = None
    ain_pot_raw = None
    ain_light_raw = None
    ain_aux_raw = None
    temp_inside_C = 0
    temp_inside_F = 0
    temp_outside_C = 0
    temp_outside_F = 0
    post_data = None
    message = ""

    def __init__(self): 
        #use factory methond in IoTGreenhouse Service
        pass

    



#MIT License

#Copyright (c) 2016 Zoltan Szarvas

#Permission is hereby granted, free of charge, to any person obtaining a copy
#of this software and associated documentation files (the "Software"), to deal
#in the Software without restriction, including without limitation the rights
#to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
#copies of the Software, and to permit persons to whom the Software is
#furnished to do so, subject to the following conditions:

#The above copyright notice and this permission notice shall be included in all
#copies or substantial portions of the Software.

#THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
#IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
#FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
#AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
#LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
#OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
#SOFTWARE.

import time
import RPi.GPIO as GPIO


class GHTempSensorResult:
    'GHTempSensor result returned by GHTempSensor.read() method'

    ERR_NO_ERROR = 0
    ERR_MISSING_DATA = 1
    ERR_CRC = 2

    error_code = ERR_NO_ERROR
    temperature = -1
    humidity = -1

    def __init__(self, error_code, temperature, humidity):
        self.error_code = error_code
        self.temperature = temperature
        self.humidity = humidity

    def is_valid(self):
        return self.error_code == GHTempSensorResult.ERR_NO_ERROR


class GHTempSensor:
    'GHTempSensor(DHT11) sensor reader class for Raspberry'

    __pin = 0
    
    def __init__(self, pin):
        self.__pin = pin

    def read(self):
        GPIO.setwarnings(False)
        GPIO.setmode(GPIO.BOARD)
        GPIO.setup(self.__pin, GPIO.OUT)

        # send initial high
        self.__send_and_sleep(GPIO.HIGH, 0.05)

        # pull down to low
        self.__send_and_sleep(GPIO.LOW, 0.02)

        # change to input using pull up
        GPIO.setup(self.__pin, GPIO.IN, GPIO.PUD_UP)

        # collect data into an array
        data = self.__collect_input()

        # parse lengths of all data pull up periods
        pull_up_lengths = self.__parse_data_pull_up_lengths(data)

        # if bit count mismatch, return error (4 byte data + 1 byte checksum)
        if len(pull_up_lengths) != 40:
            return GHTempSensorResult(GHTempSensorResult.ERR_MISSING_DATA, 0, 0)

        # calculate bits from lengths of the pull up periods
        bits = self.__calculate_bits(pull_up_lengths)

        # we have the bits, calculate bytes
        the_bytes = self.__bits_to_bytes(bits)

        # calculate checksum and check
        checksum = self.__calculate_checksum(the_bytes)
        if the_bytes[4] != checksum:
            return GHTempSensorResult(GHTempSensorResult.ERR_CRC, 0, 0)

        # ok, we have valid data, return it
        return GHTempSensorResult(GHTempSensorResult.ERR_NO_ERROR, the_bytes[2], the_bytes[0])

    def __send_and_sleep(self, output, sleep):
        GPIO.output(self.__pin, output)
        time.sleep(sleep)

    def __collect_input(self):
        # collect the data while unchanged found
        unchanged_count = 0

        # this is used to determine where is the end of the data
        max_unchanged_count = 100

        last = -1
        data = []
        while True:
            current = GPIO.input(self.__pin)
            data.append(current)
            if last != current:
                unchanged_count = 0
                last = current
            else:
                unchanged_count += 1
                if unchanged_count > max_unchanged_count:
                    break

        return data

    def __parse_data_pull_up_lengths(self, data):
        STATE_INIT_PULL_DOWN = 1
        STATE_INIT_PULL_UP = 2
        STATE_DATA_FIRST_PULL_DOWN = 3
        STATE_DATA_PULL_UP = 4
        STATE_DATA_PULL_DOWN = 5

        state = STATE_INIT_PULL_DOWN

        lengths = [] # will contain the lengths of data pull up periods
        current_length = 0 # will contain the length of the previous period

        for i in range(len(data)):

            current = data[i]
            current_length += 1

            if state == STATE_INIT_PULL_DOWN:
                if current == GPIO.LOW:
                    # ok, we got the initial pull down
                    state = STATE_INIT_PULL_UP
                    continue
                else:
                    continue
            if state == STATE_INIT_PULL_UP:
                if current == GPIO.HIGH:
                    # ok, we got the initial pull up
                    state = STATE_DATA_FIRST_PULL_DOWN
                    continue
                else:
                    continue
            if state == STATE_DATA_FIRST_PULL_DOWN:
                if current == GPIO.LOW:
                    # we have the initial pull down, the next will be the data pull up
                    state = STATE_DATA_PULL_UP
                    continue
                else:
                    continue
            if state == STATE_DATA_PULL_UP:
                if current == GPIO.HIGH:
                    # data pulled up, the length of this pull up will determine whether it is 0 or 1
                    current_length = 0
                    state = STATE_DATA_PULL_DOWN
                    continue
                else:
                    continue
            if state == STATE_DATA_PULL_DOWN:
                if current == GPIO.LOW:
                    # pulled down, we store the length of the previous pull up period
                    lengths.append(current_length)
                    state = STATE_DATA_PULL_UP
                    continue
                else:
                    continue

        return lengths

    def __calculate_bits(self, pull_up_lengths):
        # find shortest and longest period
        shortest_pull_up = 1000
        longest_pull_up = 0

        for i in range(0, len(pull_up_lengths)):
            length = pull_up_lengths[i]
            if length < shortest_pull_up:
                shortest_pull_up = length
            if length > longest_pull_up:
                longest_pull_up = length

        # use the halfway to determine whether the period it is long or short
        halfway = shortest_pull_up + (longest_pull_up - shortest_pull_up) / 2
        bits = []

        for i in range(0, len(pull_up_lengths)):
            bit = False
            if pull_up_lengths[i] > halfway:
                bit = True
            bits.append(bit)

        return bits

    def __bits_to_bytes(self, bits):
        the_bytes = []
        byte = 0

        for i in range(0, len(bits)):
            byte = byte << 1
            if (bits[i]):
                byte = byte | 1
            else:
                byte = byte | 0
            if ((i + 1) % 8 == 0):
                the_bytes.append(byte)
                byte = 0

        return the_bytes

    def __calculate_checksum(self, the_bytes):
        return the_bytes[0] + the_bytes[1] + the_bytes[2] + the_bytes[3] & 255



import setuptools
from pkg_resources import Requirement, resource_filename

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="iot_gh",
    version="0.2.4",
    author="Keith E. Kelly",
    author_email="kkelly@k2controls.net",
    description="IoT Greenhouse service",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/pypa/sampleprojecthttps://github.com/k2controls/iot_gh.git",
    packages=setuptools.find_packages(),
    package_data = {"": ["*.conf"]},
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Topic :: Education",
        "Development Status :: 3 - Alpha"
    ]
  
)
